<?php
namespace Zeedhi\DevOps\Model\Devops\Repositories;

use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;


class ProductVersion extends \Doctrine\ORM\EntityRepository {

    public function findProductVersionWithConcatVersionByProduct($productId,$organizationId, $limit = 500, $offset = 1, $filterString = '') {
      
        $query = <<<SQL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1 
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findLtsVersionWithQuantityFilter($productId, $quantity, $organizationId) {
        $query = <<<SQL
            SELECT *
              FROM (SELECT A.*, ROWNUM AS ROW_NUMBER
                      FROM (SELECT PR.PRODUCT_ID, PR.ID, PR.VERSION, PR.REVISION
                              FROM PRODUCT_VERSION PR
                             WHERE PR.PRODUCT_ID = :PRODUCT
                               AND PR.ORGANIZATION_ID = :ORGANIZATION
                               AND PR.IS_LTS = :IS_LTS
                          ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC) A)
            WHERE ROW_NUMBER > :QUANTITY
SQL;
        $params = array(
            'PRODUCT'      => $productId,
            'ORGANIZATION' => $organizationId,
            'QUANTITY'     => $quantity,
            'IS_LTS'       => \Zeedhi\DevOps\Model\Devops\Entities\ProductVersion::IS_LTS_YES,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMaxProductVersionWithFilter($productId, $filter) {
        $query = <<<SQL
			SELECT *
              FROM (SELECT PR.ID AS PRODUCT_VERSION_ID, (PR.VERSION || '.' || PR.REVISION  || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                           $filter
                     ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
             WHERE ROWNUM = 1
SQL;

        $params = array( 'PRODUCT' => $productId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductVersionConcatenated($productId, $organizationId, $limit = 500, $offset = 0, $filterString = '')
    {
      
        $query = <<<SQL
            SELECT *
              FROM (SELECT NULL AS PRODUCT_ID, NULL AS ID, 'Última versão' AS VERSION
                      FROM DUAL)
             WHERE 1=1
             $filterString
         UNION ALL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
             WHERE 1=1 
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductVersionLtsConcatenatedWithoutBuild($productId, $organizationId, $limit = 500, $offset = 0, $filterString = '', $filterNotLTS = 0)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION, 
                           DECODE(PR.IS_LTS, 'Y', 'LTS', 'Não LTS') AS IS_LTS
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND (PR.IS_LTS = 'Y' OR PR.ID IN ($filterNotLTS))
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1 
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findLtsCurrentProductVersionFromVersion($productId, $organizationId, $version, $revision, $productVersionId = false)
    {
        $productVersionIdFilter = "";
        if($productVersionId) {
            $productVersionIdFilter = "AND PR.ID <> $productVersionId";
        }
        $query = <<<SQL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.IS_LTS = 'Y'
                       AND (PR.VERSION > :VERSION 
                            OR (PR.VERSION = :VERSION AND PR.REVISION >= :REVISION)
                           )
                       $productVersionIdFilter
                  ORDER BY PR.VERSION ASC, PR.REVISION ASC)
            WHERE ROWNUM = 1
SQL;
        $params = array(
            'VERSION'      => $version,
            'REVISION'     => $revision,
            'PRODUCT'      => $productId,
            'ORGANIZATION' => $organizationId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);   
    }

    public function findNextVersionsLts($productId, $organizationId, $version, $revision, $productVersionId)
    {
        $query = <<<SQL
            SELECT * 
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.IS_LTS = 'Y'
                       AND (PR.VERSION > :VERSION 
                            OR (PR.VERSION = :VERSION AND PR.REVISION >= :REVISION)
                           )
                       AND PR.ID <> :PRODUCT_VERSION_ID
                  ORDER BY PR.VERSION ASC, PR.REVISION ASC)
SQL;

        $params = array(
            'VERSION'            => $version,
            'REVISION'           => $revision,
            'PRODUCT'            => $productId,
            'ORGANIZATION'       => $organizationId,
            'PRODUCT_VERSION_ID' => $productVersionId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findLtsVersionWithoutBuildWithZeroVersion($productId, $organizationId, $limit = 500, $offset = 0, $filterString = '')
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT NULL AS PRODUCT_ID, NULL AS ID, '0.0' AS VERSION
                      FROM DUAL)
             WHERE 1=1
             $filterString
         UNION ALL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, PR.ID, (PR.VERSION || '.' || PR.REVISION) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.IS_LTS = 'Y'
                  ORDER BY PR.VERSION DESC, PR.REVISION DESC, PR.BUILD DESC)
            WHERE 1=1
            $filterString
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCurrentBuildFromVersion($productId, $organizationId, $version, $revision)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT PR.PRODUCT_ID, PR.ID,
                           PR.VERSION AS PR_VERSION, PR.REVISION AS PR_REVISION, PR.BUILD AS PR_BUILD,
                           (PR.VERSION || '.' || PR.REVISION || '.' || PR.BUILD) AS VERSION
                      FROM PRODUCT_VERSION PR
                     WHERE PR.PRODUCT_ID = :PRODUCT
                       AND PR.ORGANIZATION_ID = :ORGANIZATION
                       AND PR.VERSION = :VERSION
                       AND PR.REVISION = :REVISION
                  ORDER BY PR.BUILD DESC)
             WHERE ROWNUM = 1
SQL;
        $params = array(
            'PRODUCT'      => $productId,
            'ORGANIZATION' => $organizationId,
            'VERSION'      => $version,
            'REVISION'     => $revision
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCurrentRevisionVersion($productId)
    {
        $query = <<<SQL
            SELECT ID, VERSION, REVISION, PRODUCT_ID
              FROM PRODUCT_VERSION
             WHERE PRODUCT_ID = :PRODUCT_ID
               AND IS_REVISION = 'Y'
             ORDER BY VERSION DESC, REVISION DESC
             FETCH FIRST 1 ROW ONLY
SQL;
        $params = array (
            'PRODUCT_ID'   => $productId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findLTSProductVersionToRemove($productId, $filterDate)
    {
        $query = <<<SQL
            SELECT ID, (VERSION || '.' || REVISION) AS VERSION, TO_CHAR(CREATED, 'DD/MM/YYYY') AS CREATION_DATE
              FROM PRODUCT_VERSION PV
             WHERE IS_LTS = 'Y'
               AND PRODUCT_ID = :PRODUCT_ID
               AND TRUNC(CREATED) <= TO_DATE(:FILTER_DATE, 'DD/MM/YYYY')
             ORDER BY VERSION DESC, REVISION DESC
SQL;
        $params = array (
            'PRODUCT_ID'  => $productId,
            'FILTER_DATE' => $filterDate,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
     public function versionWithRelationshipWithHomologatedSubRelease($productVersionIds)
    {
        $query = <<<SQL
            SELECT PV.ID, (PV.VERSION || '.' || PV.REVISION) AS VERSION
              FROM PRODUCT_VERSION  PV
              JOIN SUBRELEASE SR ON SR.PRODUCT_VERSION_ID = PV.ID 
                                AND SR.STATUS <> 'H'
                                AND SR.IS_ACTIVE = 'Y'
             WHERE PV.ID IN ($productVersionIds)
          GROUP BY PV.ID, PV.VERSION, PV.REVISION
SQL;
        
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
}